@@ -8,12 +8,13 @@ from astropy import units as u
 import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib.ticker import AutoLocator
-from scipy.optimize import curve_fit
+from scipy.optimize import curve_fit, OptimizeWarning
 from scipy.stats import gaussian_kde
 import emcee
 import corner
 import os
 from pkg_resources import resource_filename
+import warnings
 
 for filt in all_filters:
     filt.read_curve()
@@ -332,8 +333,11 @@ def blackbody_lstsq(epoch1, z, p0=None, T_range=(1., 100.), R_range=(0.01, 1000.
     def planck_cutoff(nu, T, R):
         return planck_fast(nu, T, R, cutoff_freq)
 
-    p0, cov = curve_fit(planck_cutoff, epoch1['freq'] * (1. + z), epoch1['lum'], p0=p0,
-                        bounds=([T_range[0], R_range[0]], [T_range[1], R_range[1]]))
+    with warnings.catch_warnings():
+        if len(epoch1) <= 2:
+            warnings.simplefilter('ignore', OptimizeWarning)
+        p0, cov = curve_fit(planck_cutoff, epoch1['freq'] * (1. + z), epoch1['lum'], p0=p0,
+                            bounds=([T_range[0], R_range[0]], [T_range[1], R_range[1]]))
     temp, radius = p0
     dtemp, drad = np.sqrt(np.diag(cov))
     lum, dlum = stefan_boltzmann(temp, radius, dtemp, drad, cov[0, 1])
@@ -412,7 +416,7 @@ def calc_colors(epoch1, colors):
     return mags, dmags, lolims, uplims
 
 
-def plot_color_curves(t, colors=None, fmt='o', limit_length=0.1):
+def plot_color_curves(t, colors=None, fmt='o', limit_length=0.1, xcol='MJD'):
     """
     Plot the color curves calculated by :func:`calculate_bolometric`.
 
@@ -435,13 +439,18 @@ def plot_color_curves(t, colors=None, fmt='o', limit_length=0.1):
     if colors is None:
         colors = []
         for col in t.colnames:
-            if col.split('-')[0] in filtdict and not t[col].mask.all():
+            if col.split('-')[0] in filtdict and not (t.has_masked_values and t.mask[col].all()):
                 colors.append(col)
     fig = plt.figure()
     for c in colors:
-        plt.errorbar(t['MJD'], t[c], t[f'd({c})'].filled(limit_length), (t['dMJD0'], t['dMJD1']), fmt=fmt,
+        dcolor_colname = f'd({c})'
+        if t.has_masked_values and t.mask[dcolor_colname].any():
+            dcolor = t[dcolor_colname].filled(limit_length)
+        else:
+            dcolor = t[dcolor_colname]
+        plt.errorbar(t[xcol], t[c], dcolor, (t[f'd{xcol}0'], t[f'd{xcol}1']), fmt=fmt,
                      lolims=t[f'lolims({c})'], uplims=t[f'uplims({c})'], label=f'${c}$')
-    plt.xlabel('MJD')
+    plt.xlabel(xcol)
     plt.ylabel('Color (mag)')
     plt.legend()
     return fig
@@ -493,6 +502,10 @@ def calculate_bolometric(lc, z, outpath='.', res=1., nwalkers=10, burnin_steps=2
         Table containing the blackbody parameters, bolometric luminosities, and (optionally) colors
     """
 
+    if colors is None:
+        colors = []
+
+    use_src = 'source' in lc.colnames
     t0 = LC(names=['MJD', 'dMJD0', 'dMJD1',
                    'temp', 'radius', 'dtemp', 'dradius',  # best fit from scipy.curve_fit
                    'lum', 'dlum',  # total bolometric luminosity from scipy.curve_fit
@@ -502,14 +515,16 @@ def calculate_bolometric(lc, z, outpath='.', res=1., nwalkers=10, burnin_steps=2
                    'L_int',  # pseudobolometric luminosity from direct integration of the SED
                    'npoints']
             + colors + ['d({})'.format(c) for c in colors] + ['lolims({})'.format(c) for c in colors]
-            + ['uplims({})'.format(c) for c in colors] + ['filts', 'source'],
+            + ['uplims({})'.format(c) for c in colors] + ['filts'] + (['source'] if use_src else []),
             dtype=[float, float, float, float, float, float, float, float, float, float, float, float, float, float,
                    float, float, float, float, float, float, int]
-            + [float] * 2 * len(colors) + [bool] * 2 * len(colors) + ['S6', lc['source'].dtype],
+            + [float] * 2 * len(colors) + [bool] * 2 * len(colors) + ['S6'] + ([lc['source'].dtype] if use_src else []),
             masked=True)
 
     sampler = None
+    lc = lc[np.isfinite(lc['dmag']) & (lc['dmag'] > 0.)]
     for epoch1 in group_by_epoch(lc, res):
+        epoch1.sn = lc.sn
         epoch1.calcFlux()
         epoch1 = epoch1.bin(delta=np.inf)
         epoch1.calcMag()
@@ -572,11 +587,12 @@ def calculate_bolometric(lc, z, outpath='.', res=1., nwalkers=10, burnin_steps=2
                T_mcmc, R_mcmc, dT0_mcmc, dT1_mcmc, dR0_mcmc, dR1_mcmc, L_mcmc, dL_mcmc0, dL_mcmc1,
                L_int, nfilt] + color_mags + color_dmags
         row_bool = color_lolims + color_uplims
-        row_string = [filtstr, epoch1['source'][0]]
-        mask = np.concatenate([np.isnan(row), np.zeros_like(row_bool), ~np.array(row_string, dtype=bool)])
+        row_string = [filtstr] + ([epoch1['source'][0]] if use_src else [])
+        mask = np.concatenate([np.isnan(row), np.zeros_like(row_bool, dtype=bool),
+                               ~np.array([bool(rs) for rs in row_string])])
         t0.add_row(row + row_bool + row_string, mask=mask)
 
-    if save_table_as is not None:
-        t0.write(save_table_as, format='ascii.fixed_width', overwrite=True)
+    if save_table_as is not None and t0:
+        t0.write(save_table_as, format='ascii.fixed_width_two_line', overwrite=True)
 
     return t0
