@@ -1,7 +1,7 @@
 import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib.path import Path
-from astropy.table import Table, vstack
+from astropy.table import Table, vstack, MaskedColumn
 from .filters import filtdict
 import itertools
 from matplotlib.markers import MarkerStyle
@@ -32,6 +32,27 @@ othermarkers = ('o', *MarkerStyle.filled_markers[2:])
 itermarkers = itertools.cycle(othermarkers)
 usedmarkers = []
 
+# if you edit this list, also add the new names to usage.rst
+column_names = {
+    'Filter': ['filt', 'filter', 'Filter', 'band', 'FLT', 'Band'],
+    'Telescope': ['telescope', 'Telescope', 'Tel', 'tel+inst'],
+    'Source': ['source', 'Source'],
+    'Apparent Magnitude': ['mag', 'Magnitude', 'Mag', 'ab_mag', 'PSFmag', 'MAG', 'omag', 'magnitude', 'apparent_mag'],
+    'Apparent Magnitude Uncertainty': [
+        'dmag', 'Magnitude_Error', 'magerr', 'MagErr', 'mag_err', 'e_mag', 'Error', 'err', 'PSFerr', 'MAGERR', 'e_omag',
+        'e_magnitude', 'apparent_mag_err', 'Mag_Err',
+    ],
+    'MJD': ['MJD', 'mjd'],
+    'JD': ['JD', 'jd'],
+    'Phase (rest days)': ['phase', 'Phase', 'PHASE'],
+    'Flux $F_ν$ (W m$^{-2}$ Hz$^{-1}$)': ['flux', 'FLUXCAL'],
+    'Flux Uncertainty': ['dflux', 'FLUXCALERR'],
+    'Nondetection': ['nondet', 'Is_Limit', 'UL', 'l_omag', 'upper_limit', 'upperlimit'],
+    'Absolute Magnitude': ['absmag'],
+    'Luminosity $L_ν$ (W Hz$^{-1}$)': ['lum'],
+    'Luminosity Uncertainty': ['dlum'],
+}
+
 
 class LC(Table):
     """
@@ -51,6 +72,7 @@ class LC(Table):
         self.sn = None
         self.nondetSigmas = 3.
         self.groupby = {'filt', 'filter', 'source'}
+        self.markers = markers.copy()
 
     def where(self, **kwargs):
         """
@@ -95,8 +117,32 @@ class LC(Table):
         selected = self[use]
         selected.sn = self.sn
         selected.meta = self.meta
+        selected.markers = self.markers
         return selected
 
+    def get(self, key, default=None):
+        return self[key] if key in self.colnames else default
+
+    def normalize_column_names(self):
+        """
+        Rename any recognizable columns to their standard names for this package (see `lightcurve.column_names`).
+        """
+        for good_key, *bad_keys in column_names.values():
+            if good_key not in self.colnames:
+                for bad_key in bad_keys:
+                    if bad_key in self.colnames:
+                        self.rename_column(bad_key, good_key)
+                        break
+        if 'MJD' not in self.colnames and 'JD' in self.colnames:
+            self['MJD'] = self['JD'] - 2400000.5
+            self.remove_column('JD')
+        if 'nondet' in self.colnames and self['nondet'].dtype != bool:
+            if isinstance(self['nondet'], MaskedColumn):
+                self['nondet'] = self['nondet'].filled()
+            nondet = (self['nondet'] == 'True') | (self['nondet'] == 'T') | (self['nondet'] == '>')
+            self.remove_column('nondet')
+            self['nondet'] = nondet
+
     def filters_to_objects(self, read_curve=True):
         """
         Parse the ``'filt'`` column into :class:`filters.Filter` objects and store in the ``'filter'`` column
@@ -106,12 +152,14 @@ class LC(Table):
         read_curve : bool, optional
             Read in the transmission function for each filter encountered (default)
         """
-        self['filter'] = [filtdict[str(f)] for f in self['filt']]
+        self['filter'] = [filtdict['?'] if np.ma.is_masked(f) or str(f) not in filtdict else filtdict[str(f)]
+                          for f in self['filt']]
         is_swift = np.zeros(len(self), bool)
         if 'telescope' in self.colnames:
             is_swift |= self['telescope'] == 'Swift'
             is_swift |= self['telescope'] == 'UVOT'
             is_swift |= self['telescope'] == 'Swift/UVOT'
+            is_swift |= self['telescope'] == 'Swift+UVOT'
         if 'source' in self.colnames:
             is_swift |= self['source'] == 'SOUSA'
         if is_swift.any():
@@ -143,7 +191,7 @@ class LC(Table):
             self.nondetSigmas = nondetSigmas
         if zp is None:
             zp = self.zp
-        self['flux'], self['dflux'] = mag2flux(self['mag'], self['dmag'], zp, self['nondet'], self.nondetSigmas)
+        self['flux'], self['dflux'] = mag2flux(self['mag'], self['dmag'], zp, self.get('nondet'), self.nondetSigmas)
 
     def bin(self, delta=0.3, groupby=None):
         """
@@ -209,7 +257,7 @@ class LC(Table):
         self.findNondet()
         if zp is None:
             zp = self.zp
-        self['mag'], self['dmag'] = flux2mag(self['flux'], self['dflux'], zp, self['nondet'], self.nondetSigmas)
+        self['mag'], self['dmag'] = flux2mag(self['flux'], self['dflux'], zp, self.get('nondet'), self.nondetSigmas)
 
     def calcAbsMag(self, dm=None, extinction=None, hostext=None):
         """
@@ -265,7 +313,7 @@ class LC(Table):
         """
         if nondetSigmas is not None:
             self.nondetSigmas = nondetSigmas
-        self['lum'], self['dlum'] = mag2flux(self['absmag'], self['dmag'], self.zp + 90.19, self['nondet'],
+        self['lum'], self['dlum'] = mag2flux(self['absmag'], self['dmag'], self.zp + 90.19, self.get('nondet'),
                                              self.nondetSigmas)
 
     def findPeak(self, **criteria):
@@ -306,22 +354,28 @@ class LC(Table):
         rdsp : bool, optional
             Define phase as rest-frame days since peak, rather than rest-frame days since explosion
         """
-        if rdsp and self.sn.peakdate is None:
-            raise Exception('must run sn.findPeak() first')
-        elif rdsp:
-            self.sn.refmjd = self.sn.peakdate
-        elif self.sn.explosion is not None:
-            self.sn.refmjd = self.sn.explosion
-        else:
-            if 'nondet' in self.colnames:
-                detections = self.where(nondet=False)
+        if 'refmjd' not in self.meta and self.sn is not None:
+            if rdsp and self.sn.peakdate is None:
+                raise Exception('must run sn.findPeak() first')
+            elif rdsp:
+                self.sn.refmjd = self.sn.peakdate
+            elif self.sn.explosion is not None:
+                self.sn.refmjd = self.sn.explosion
             else:
-                detections = self
-            self.sn.refmjd = np.min(detections['MJD'].data)
-        phase = (self['MJD'].data - self.sn.refmjd) / (1 + self.sn.z)
-        self['phase'] = phase
+                if 'nondet' in self.colnames:
+                    detections = self.where(nondet=False)
+                else:
+                    detections = self
+                self.sn.refmjd = np.min(detections['MJD'].data)
+            self.meta['refmjd'] = self.sn.refmjd
+            self.meta['redshift'] = self.sn.z
+        self['phase'] = (self['MJD'].data - self.meta['refmjd']) / (1 + self.meta['redshift'])
+        if 'dMJD0' in self.colnames:
+            self['dphase0'] = self['dMJD0'] / (1. + self.meta['redshift'])
+        if 'dMJD1' in self.colnames:
+            self['dphase1'] = self['dMJD1'] / (1. + self.meta['redshift'])
 
-    def plot(self, xcol='phase', ycol='absmag', offset_factor=1., color='filter', marker='source', use_lines=False,
+    def plot(self, xcol='phase', ycol='absmag', offset_factor=1., color='filter', marker=None, use_lines=False,
              normalize=False, fillmark=True, **kwargs):
         """
         Plot the light curve, with nondetections marked with a downward-pointing arrow
@@ -337,7 +391,7 @@ class LC(Table):
         color : str, optional
             Column that controls the color of the lines and points. Default: ``'filter'``
         marker : str, optional
-            Column that controls the marker shape. Default: ``'source'``
+            Column that controls the marker shape. Default: ``'source'`` or ``'telescope'``
         use_lines : bool, optional
             Connect light curve points with lines. Default: False
         normalize : bool, optional
@@ -348,7 +402,6 @@ class LC(Table):
             Keyword arguments matching column names in the light curve are used to specify a subset of points to plot.
             Additional keyword arguments passed to :func:`matplotlib.pyplot.plot`.
         """
-        global markers
         xchoices = ['phase', 'MJD']
         while xcol not in self.keys():
             xchoices.remove(xcol)
@@ -363,6 +416,13 @@ class LC(Table):
                 ycol = ychoices[0]
             else:
                 raise Exception('no columns found for y-axis')
+        if marker is None:
+            if 'source' in self.colnames:
+                marker = 'source'
+            elif 'telescope' in self.colnames:
+                marker = 'telescope'
+            else:
+                marker = 'o'
         plotthese = np.tile(True, len(self))
         criteria = {key: val for key, val in kwargs.items() if key in self.colnames}
         plot_kwargs = {key: val for key, val in kwargs.items() if key not in self.colnames}
@@ -400,20 +460,20 @@ class LC(Table):
             if marker == 'name':
                 mark = self.sn.marker
             elif marker in plottable.keys():
-                if g[marker][0] not in markers:
+                if g[marker][0] not in self.markers:
                     for nextmarker in othermarkers:
                         if nextmarker not in usedmarkers:
-                            markers[g[marker][0]] = nextmarker
+                            self.markers[g[marker][0]] = nextmarker
                             break
                     else:
                         for nextmarker in itermarkers:
-                            markers[g[marker][0]] = nextmarker
+                            self.markers[g[marker][0]] = nextmarker
                             break
-                mark = markers[g[marker][0]]
-            elif not marker:
-                mark = None
-            else:
+                mark = self.markers[g[marker][0]]
+            elif marker in MarkerStyle.markers:
                 mark = marker
+            else:
+                mark = MarkerStyle.markers[0]
             usedmarkers.append(mark)
             if use_lines:
                 g.sort(xcol)
@@ -429,12 +489,16 @@ class LC(Table):
                 y -= self.sn.peakabsmag
             if 'mag' in ycol and 'nondet' in g.keys() and marker:  # don't plot if no markers used
                 plt.plot(x[g['nondet']], y[g['nondet']], marker=arrow, linestyle='none', ms=25, mec=mec, **plot_kwargs)
+            if len(filt.name) >= 4 and not filt.offset:
+                label = filt.name
+            elif offset_factor:
+                label = '${}{:+.0f}$'.format(filt.name, -filt.offset * offset_factor)
+            else:
+                label = '${}$'.format(filt.name)
             if self.sn is None:
-                label = None
                 linestyle = None
                 linewidth = None
             else:
-                label = self.sn.name
                 linestyle = self.sn.linestyle
                 linewidth = self.sn.linewidth
             if not use_lines:
@@ -451,19 +515,18 @@ class LC(Table):
         ymin, ymax = plt.ylim()
         if 'mag' in ycol and ymax > ymin:
             plt.ylim(ymax, ymin)
+        for axlabel, keys in column_names.items():
+            if xcol in keys:
+                plt.xlabel(axlabel)
+            elif ycol in keys:
+                plt.ylabel(axlabel)
 
     @classmethod
-    def read(cls, filepath, format='ascii.fixed_width', fill_values=None, **kwargs):
+    def read(cls, filepath, format='ascii', fill_values=None, **kwargs):
         if fill_values is None:
             fill_values = [('--', '0'), ('', '0')]
         t = super(LC, cls).read(filepath, format=format, fill_values=fill_values, **kwargs)
-        # make nondetections booleans
-        if 'nondet' in t.keys():
-            nondets = t['nondet'] == 'True'
-            t.remove_column('nondet')
-            t['nondet'] = nondets
-        else:
-            t['nondet'] = False
+        t.normalize_column_names()
         if 'filt' in t.colnames:
             t.filters_to_objects()
         return t
@@ -535,7 +598,7 @@ def mag2flux(mag, dmag=np.nan, zp=0., nondet=None, nondetSigmas=3.):
     return flux, dflux
 
 
-def binflux(time, flux, dflux, delta=0.2):
+def binflux(time, flux, dflux, delta=0.2, include_zero=True):
     """
     Bin a light curve by averaging points within ``delta`` of each other in time
 
@@ -545,6 +608,8 @@ def binflux(time, flux, dflux, delta=0.2):
         Arrays of times, fluxes, and uncertainties comprising the observed light curve
     delta : float, optional
         Bin size, in the same units as ``time``. Default: 0.2
+    include_zero : bool, optional
+        Include data points with no error bar
 
     Returns
     -------
@@ -559,18 +624,30 @@ def binflux(time, flux, dflux, delta=0.2):
         time_grp = time[grp]
         flux_grp = flux[grp]
         dflux_grp = dflux[grp]
-        if any(dflux_grp == 0) or any(dflux_grp == 9999) or any(np.isnan(dflux_grp)) or (
-                np.ma.is_masked(dflux_grp) and any(dflux_grp.mask)):
+
+        # Indices with no error bar
+        zeros = (dflux_grp == 0) | (dflux_grp == 999) | (dflux_grp == 9999) | (dflux_grp == -1) | np.isnan(dflux_grp)
+        if np.ma.is_masked(dflux_grp):
+            zeros = zeros.data | dflux_grp.mask
+
+        if any(zeros) and include_zero:
             x = np.mean(time_grp)
             y = np.mean(flux_grp)
             z = 0.
         else:
+            # Remove points with no error bars
+            time_grp = time_grp[~zeros]
+            flux_grp = flux_grp[~zeros]
+            dflux_grp = dflux_grp[~zeros]
+
             x = np.mean(time_grp)
             y = np.sum(flux_grp * dflux_grp ** -2) / np.sum(dflux_grp ** -2)
             z = np.sum(dflux_grp ** -2) ** -0.5
-        bin_time.append(x)  # indent these lines
-        bin_flux.append(y)  # to exlude points
-        bin_dflux.append(z)  # with no error
+        # Append result 
+        bin_time.append(x)
+        bin_flux.append(y)
+        bin_dflux.append(z)
+        # Remove data points already used
         time = time[~grp]
         flux = flux[~grp]
         dflux = dflux[~grp]
